#include "Terrain/ClipmapSampling.hlsl"

// Input height maps
Texture2D heightTexture;
Texture2D coarseHeightTexture;

SamplerState heightSampler;
SamplerState coarseHeightSampler;

Texture2D imageryTexture[NUM_TEXTURES];
Texture2D coarseImageryTexture[NUM_TEXTURES];

SamplerState normalSampler;
SamplerState imagerySampler;

float4 sampleTexture(in Texture2D source, float2 texCoords, float2 texelOffset)
{
  float2 dimensions;
  source.GetDimensions(dimensions.x, dimensions.y);
  float2 invDimensions = 1.0 / dimensions;

  return source.SampleLevel(imagerySampler, texCoords + texelOffset * invDimensions, 0);
}

float4 sampleBlended(in Texture2D fineSource, in Texture2D coarseSource, in float4 texCoords, in float2 fineOffset, in float2 coarseOffset, in float blendFactor)
{
  float4 fineColor = sampleTexture(fineSource, texCoords.xy, fineOffset);
  float4 coarseColor = sampleTexture(coarseSource, texCoords.zw, coarseOffset);

  return lerp(fineColor, coarseColor, blendFactor);
}

bool testClipping(in VertexIn In)
{
  if (In.WorldPosition.x < (worldExtents.x + worldOffset.x) || In.WorldPosition.x >(worldExtents.z + worldOffset.x) ||
    In.WorldPosition.y < (worldExtents.y + worldOffset.y) || In.WorldPosition.y >(worldExtents.w + worldOffset.y)) {
    return false;
  }

  if (enableNoData) {
    if (In.NodataFactor != 0) return false;
  }

  return true;
}
