#include "Terrain/ConstantBuffers.hlsl"
#include "Terrain/ClipmapCommonVS.hlsl"

float4 calculateImageryCoordinates(in float2 levelPos, in int i)
{
  float4 result = float4(0, 0, 0, 0);

  float2 imageryLevelPos = levelPos * terrainToImageryResolutionRatio[i];
  result.xy = convertTexCoords((imageryLevelPos + terrainOffsetInImagery[i]) * oneOverImagerySize[i]);
  result.zw = convertTexCoords(((imageryLevelPos * fineToCoarseImageryResolutionRatio[i]) + fineImageryOriginInCoarse[i]) * oneOverCoarseImagerySize[i]);

  return result;
}

VertexOut vertexFunction(VertexIn In)
{
  VertexOut Out = (VertexOut)0;

  float2 levelPos = In.Position.xy + patchOriginInClippedLevel;
  float2 clipmapPos = (levelPos + levelOffsetFromClipmapCenter) * levelScaleFactor * levelZeroWorldScaleFactor;
  float2 worldPos = clipmapPos * worldScale.xy + centerOffsetFromOrigin;

  [unroll(NUM_ACTIVE_TEXTURES)]
  for (int i = 0; i < NUM_ACTIVE_TEXTURES; ++i) {
    Out.ImageryTexCoords[i] = calculateImageryCoordinates(levelPos, i);
  }

  // Add small offset to allow discard to work after clamping.
  float offset = 0.1;

  float x = clamp(worldPos.x, worldExtents.x - offset, worldExtents.z + offset);
  float y = clamp(worldPos.y, worldExtents.y - offset, worldExtents.w + offset);

  float3 displacedPosition = float3(x, y, sampleHeight(Out, levelPos) * heightExaggeration) + worldOffset.xyz;

  Out.Position = mul(float4(displacedPosition, 1.0), worldToClipMatrix);
  Out.WorldPosition = float4(displacedPosition, 1.0);
  Out.ClipDistance = applyClipping(Out.WorldPosition - worldOffset);

  return Out;
}
