#include "Terrain/ConstantBuffers.hlsl"
#include "Terrain/Noise.hlsl"
#include "Terrain/OceanCommon.hlsl"

float fresnel(float3 view, float3 normal)
{
  float refractionIndexRatio = 1.33;
  float R0 = pow(1.0 - refractionIndexRatio, 2.0) / pow(1.0 + refractionIndexRatio, 2.0);
  float cosAngle = 1.0 - clamp(dot(view, normal), 0, 1);

  float result = cosAngle * cosAngle;
  result = result * result;
  result = result * cosAngle;
  result = clamp((result * 1 - clamp(R0, 0, 1)) + R0, 0, 1);

  return result;
}

float longitude(float2 d)
{
  float2 planeDir = normalize(-d);
  float2 camPlaneDir = float2(0, 1);
  float camAzimuth = 0;

  float camRelAzimuth = (0.5 / 3.14159) * acos(min(1.0, dot(planeDir, camPlaneDir)));

  if (planeDir.y * camPlaneDir.x - planeDir.x * camPlaneDir.y > 0.0) {
    camRelAzimuth = -camRelAzimuth;
  }

  return camRelAzimuth + camAzimuth;
}

SurfaceOut surfaceFunction(in VertexIn In)
{
  SurfaceOut Out = (SurfaceOut)0;

  float lod = In.OceanPosition.z;

  float foam = 0.0;
  float3 normal = waveNormal(foam, In.OceanPosition.xy, In.Seabed.w, In.Seabed.xyz, lod, roughness);

  float3 viewDir = normalize(inverseViewMatrix[3].xyz - In.WorldPosition.xyz);

  float fresnelTerm = fresnel(viewDir, normal);

  //float depthFade = enableWaveFade ? smoothstep(deepWaterThreshold, shallowWaterThreshold, In.OceanFactors.x) : 0.0;
  //float waveFactor = enableWaveFade ? 1.0 - depthFade * waveFade : 1.0;

  //float4 oceanColor = lerp(deepColor, shallowColor, depthFade);
  float4 oceanColor = deepColor;

  float factor = -1.0;

#ifdef COGS_SHADER_GL
  factor = 1.0;
#endif

  float2 screenPosition = float2(0.5, factor * 0.5) * (In.ScreenPosition.xy / In.ScreenPosition.w) + 0.5;

  float4 reflectedColor = PlanarReflection.Sample(PlanarReflectionSampler, screenPosition + (normal.xy * 0.05)) * float4(0.7, 0.7, 0.7, 1.0);

  float4 waterColor = float4(lerp(pow(abs(shallowColor.rgb), 2.2), pow(abs(deepColor.rgb), 2.2), In.Seabed.w), lerp(shallowColor.a, deepColor.a, In.Seabed.w));

  Out.Color = float4(waterColor.rgb * (1 - fresnelTerm), waterColor.a);

  float3 specular = float3(0.5, 0.5, 0.5);
  if (roughness > 0.0) {
    Out.Color = lerp(Out.Color, roughColor, roughness);
    Out.Color = lerp(Out.Color, float4(1, 1, 1, 1), foam + roughness*(1.0 - In.Seabed.w));
    specular = lerp(specular, float3(0.2, 0.15, 0.1), foam + roughness*(1.0 - In.Seabed.w));
  }

  Out.Emissive = (reflectedColor * fresnelTerm).rgb;
  Out.Normal = normal;
  Out.Specular = float3(0.5, 0.5, 0.5);
  Out.SpecularPower = 80;

  return Out;
}
