cbuffer DispersionParameters
{
  int N;
  float twoPiOverSideLength;
  float dt;
};

Texture2D aTex;
Texture2D phaseTex;

struct Results
{
  float phase: SV_Target0;
  float4 AXY : SV_Target1;
  float2 AZ : SV_Target2;
  float4 AdZdu_dZdv : SV_Target3;
};

float2 complexMul(float2 a, float2 b)
{
  return float2(a.x * b.x - a.y * b.y,
                a.x * b.y + a.y * b.x);
}

Results main(float4 ScreenPos : SV_Position)
{
  const float gravity = 9.80665f;
  const float twoPi = 6.28318530718;

  int2 ij = int2(ScreenPos.xy);
  float2 waveVector = twoPiOverSideLength * float2((uint)ij.x <= N / 2u ? ij.x : ij.x - N,
                                                   (uint)ij.y <= N / 2u ? ij.y : ij.y - N);

  float waveNumber = length(waveVector);

  float2 waveDir = ((ij.x == 0) && (ij.y == 0) ? 0.0f : (1.0f / waveNumber)) * waveVector;

  float angularVelocity = sqrt(gravity * waveNumber);
  float phase = angularVelocity*dt + phaseTex.Load(float3(ij, 0)).r;

  float2 A = complexMul(aTex.Load(float3(ij, 0)).rg,
                        float2(cos(phase), sin(phase)));

  Results r;
  r.phase = fmod(phase, twoPi);
  r.AXY = float4(waveDir.x * A, waveDir.y * A);
  r.AZ = A;
  r.AdZdu_dZdv = float4(waveVector.x * A, waveVector.y * A);

  return r;
}
